﻿#####################################################################################################################################################################
# Outil de récupération via FTP de la base AAF fournie et conversion en fichiers CSV
# Version mono domaine
# (C) 2006-2021 KoXo Developpement
# V1.3
#####################################################################################################################################################################
#
# Attention : 
# - Les archives fournies doivent être au format TarGZ, le nom de l'archive Tar étant supposé être le même que celui de l'archive GZ.
# - Les archives sont en principe nommées du genre "Périmètre_Complet_DATE.tar.gz", si elles sont nommées du genre "Périmètre.tar.gz" elles sont 
#   alors renommées au format contenant la date.
# Attention : Les fichiers XML doivent être au premier niveau de l'archive tar.
# - Pour supporter ce vieux format (Le Tar n'est pas géré par Powershell) il faut ajouter un module "7Z".
# - Les fichiers "7Z.dll", "7z64.dll", "7Zip4Powershell.dll", "SevenZipSharp.dll" doivent se trouver dans le sous dossier "7Z".
# - Pour utiliser le protocole SFTP, WinSCP doit être installé (fichiers "WinSCPnet.dll" et "WinSCP.exe" nécessaires)
#   --> https://winscp.net/
# - Les fichiers "AnnuFedXmlCsv.exe", "DisciplinesOuFonctions.csv", "PrenomsAccentues.txt" doivent se trouver dans le dossier courant.
#
# Les sous dossiers "Download", "Extract", "Logs", "CSV" sont créés automatiquement.
# La variable "Filter" permet d'indiquer une partie du nom des fichiers que l'on veut télécharger, si on veut tout prendre il faut laisser vide.
#
#####################################################################################################################################################################
#################################################
#                                               # 
# Valeurs à modifier : "<<--"                   #
#                                               #
#################################################
$TranfertProtocole="SFTP"                       # <<-- Protocole de transfert ; Valeurs acceptées : "FTP", "FTP/S", "SFTP", ""
                                                # Si la valeur est vide il n'y a pas de téléchargement, l'archive tar.gz doit être placée dans le dossier "Download"                          
                                                # par un autre moyen.
#################################################
$Username="admin"                               # <<-- Identifiant
$Password="Mo2P@ass"                            # <<-- Mot de passe
############ FTP et FTP/S #######################
$FTPUrl="ftp://192.168.1.203/cd74"              # <<-- Url FTP ou FTP/S
########## SFTP (Via WinSCP) ####################
$SFTPHostname="192.168.1.203"                   # <<-- Nom ou adresse IP du serveur SFTP
$SFTPPortNumber=22                              #
$SFTPFingerPrint=""                             # <<-- Empreinte SFTP ; Exemple : "ssh-rsa 2048 DCczciAwd/hUwo1lmINcMgoQyFCHkHy9tNmDSEMjtPM="
                                                # L'empreinte n'est pas forcément renseignée, elle peut être détectée automatiquement
$SFTPDetectFingerPrint=$True                    # Détection automatique de l'empreinte SFTP (conseillé)
$SFTPRemotePath="/cd75/"                        # <<-- Chemin du serveur SFTP ; Doit finir par un "/"
#################################################
$Filter=""                                      # <<-- Seuls les fichiers contenant cette chaine seront pris en compte ; Exemple : "cd94"
$AttachementStructureOnly=$False                # Indique si l'utilisateur est créé uniquement sur sa structure de rattachement
#################################################
# Paramètres dossiers, etc.
#################################################
$DownloadFolder="Download"                      #
$ExtractFolder="Extract"                        #
$CSVFolder="CSV"                                #
$LogsFolder="Logs"                              #
$FilesHistoryFileName="FilesHistory.txt"        #
$Accents=$True                                  #
$SupprimerDoublesTirets=$True                   # Remplace les doubles tirets des noms par des tirets simples (Mesure de 2005 abandonnée le 25/10/2011)
$AddDateToCSvFiles=$False                       # Permet de rajouter la date aux fichiers CSV, exemple "0750012V_Eleves_20190801.CSv"
#################################################
#####################################################################################################################################################################
# Download a folder from a FTP
# If file exists with the same size the copy is not performed
#
# Example / Usage :
# $Credentials=New-Object System.Net.NetworkCredential("Administrator","Paasssssssss") 
# $Url="ftp://ftp.test.com/myfolder"
# DownloadFtpDirectory $Url $Credentials "C:\Download" "" $False $True
#
Function DownloadFtpDirectory($Url,$Credentials,$LocalPath,$Filter,$Recurse,$EnableSSL)
{
 $Fr=((Get-Culture).LCID -eq 1036)
 $ListRequest=[System.Net.WebRequest]::Create($Url)
 $ListRequest.Credentials=$Credentials
 $ListRequest.EnableSSL=$EnableSSL
 $ListRequest.Method=[System.Net.WebRequestMethods+FTP]::ListDirectoryDetails
 If ($EnableSSL) 
    {
     [System.Net.ServicePointManager]::ServerCertificateValidationCallback={$True}
    }
 $Lines=New-Object System.Collections.ArrayList
 $ListResponse=$ListRequest.GetResponse()
 $ListStream=$ListResponse.GetResponseStream()
 $ListReader=New-Object System.IO.StreamReader($ListStream)
 While (!$ListReader.EndOfStream)
    {
     $Line=$ListReader.ReadLine()
     $Lines.Add($Line) | Out-Null
     # Line sample for Windows
     # 04-02-16 08.88:22PM 458745 file.txt
     # 04-02-16 08.88:22PM <DIR> MyFolder
     # Line sample for unix
     # -rw-r--r-- 1 admin administrators 659450 Jun 15 05:07 Test.txt
     # drwxrwxr-x 2 admin administrators 4096 May 06 12:24 MyFolder
    }
 $ListReader.Dispose()
 $ListStream.Dispose()
 $ListResponse.Dispose()
 ForEach ($Line In $Lines)
         {
          $Tokens=$Line.Split(" ",9,[StringSplitOptions]::RemoveEmptyEntries)
          # Windows (First is month) or Unix FTP ?
          If (([Byte][Char]$Line[0] -ge [Byte][Char]"0") -And ([Byte][Char]$Line[0] -le [Byte][Char]"9"))
             {
              $Name=$Tokens[3]
              $PermissionsOrSize=$Tokens[2]
             }
             else
             {
              $Name=$Tokens[8]
              $PermissionsOrSize=$Tokens[0]
             }
          $LocalFilePath = Join-Path $LocalPath $Name
          $FileUrl=($Url+"/"+$Name)
          If (($PermissionsOrSize[0] -eq 'd') -Or ($PermissionsOrSize[1] -eq 'd'))
             {
              If ($Recurse)
                 {
                  If (!(Test-Path $LocalFilePath -PathType Container))
                     {
                      If ($Fr) {$S="Création du dossier ""{0}"""} Else {$S="Creating folder ""{0}"""}
                      Write-Host ($S -f $LocalFilePath) -foreground "Green"
                      New-Item $LocalFilePath -Type Directory | Out-Null
                     }
                     else
                     {
                      If ($Fr) {$S="Le dossier ""{0}"" existe déjà !"} Else {$S="Folder ""{0}"" already exists !"}
                      Write-Host ($S -f $LocalFilePath) -foreground "Yellow"
                     }
                  ## Recurse
                  DownloadFtpDirectory ($FileUrl+"/") $Credentials $LocalFilePath $Recurse $EnableSSL
                 }
             }
             Else
             {
              # Windows (First is month) or Unix FTP ?
              If (([Byte][Char]$Line[0] -ge [Byte][Char]"0") -And ([Byte][Char]$Line[0] -le [Byte][Char]"9"))
                 {
                  $FtpFileSize=[Int]$Tokens[2]
                 }
                 else
                 {
                  $FtpFileSize=[Int]$Tokens[4] 
                 }             
              $LocalFileSize=-1
              If (Test-Path($LocalFilePath)) {$LocalFileSize=(Get-Item $LocalFilePath).Length} 
              $FilterOk=$Name.ToLower().Contains($Filter.ToLower())
              If (!$FilterOK)
                 {
                  If ($Fr) {$S="Le fichier ""{0}"" est ignoré"} Else {$S="File ""{0}"" is ignored)"}
                  Write-Host ($S -f $FileUrl) -foreground "Yellow"
                 }
              $MustCopy=(!($FtpFileSize -eq $LocalFileSize) -And $FilterOk)
              If ($MustCopy)
                 {
                  If ($Fr) {$S="Téléchargement de ""{0}"" vers ""{1}"" ({2} octets)"} Else {$S="Downloading ""{0}"" to ""{1}"" ({2} bytes)"}
                  Write-Host ($S -f $FileUrl, $LocalFilePath, $FtpFileSize) -foreground "Green"
                  $DownloadRequest=[Net.WebRequest]::Create($fileUrl)
                  $DownloadRequest.Method=[System.Net.WebRequestMethods+FTP]::DownloadFile
                  $DownloadRequest.Credentials=$Credentials
                  $DownloadResponse=$DownloadRequest.GetResponse()
                  $SourceStream=$DownloadResponse.GetResponseStream()
                  $TargetStream=[System.IO.File]::Create($LocalFilePath)
                  $Buffer=New-Object byte[] 10240
                  While (($Read=$SourceStream.Read($Buffer,0,$Buffer.Length)) -gt 0)
                        {
                         $TargetStream.Write($Buffer,0,$Read);
                        }           
                  $TargetStream.Dispose()
                  $SourceStream.Dispose()
                  $DownloadResponse.Dispose()
                 }
              If (($FtpFileSize -eq $LocalFileSize) -And $FilterOk)
                 {          
                  If ($Fr) {$S="Le fichier ""{0}"" ({1} octets) existe déjà !"} Else {$S="File ""{0}"" ({1} bytes) already exists !"}
                  Write-Host ($S -f $LocalFilePath, $LocalFileSize) -foreground "Yellow"
                 }
             }
         }
}
#####################################################################################################################################################################
# Get SFTP Server fingerprint
Function Get-SshFingerprint($ssh_server_name,$ssh_server_port)
{
 # Load WinSCP .NET assembly
 Add-Type -Path "WinSCPnet.dll"
 # Setup session options
 $sessionOptions=New-Object WinSCP.SessionOptions -Property @{
       Protocol=[WinSCP.Protocol]::Sftp
       HostName=$ssh_server_name
       PortNumber=$ssh_server_port
       UserName=""
 }
 # Scan for the host key
 $session=New-Object WinSCP.Session
 Try
    {
     $fingerprint=$session.ScanFingerprint($sessionOptions,"SHA-256")
    }
 Finally
    {
     $session.Dispose()
    }
 # And output the host key to the pipeline
 Write-Output $fingerprint
}
#####################################################################################################################################################################
# SFTP : Download files
Function DownloadSFTPDirectory($hostname,$portnumber,
                               $username,$password,
                               $detectfingerprint,$fingerprint,
                               $remotepath,$localpath,
                               $filter)
{
Try
 {
 $Fr=((Get-Culture).LCID -eq 1036)
 # Load WinSCP .NET assembly
 Add-Type -Path "WinSCPnet.dll"
 # setting session options
 $sessionOptions=New-Object WinSCP.SessionOptions -Property @{
    Protocol=[WinSCP.Protocol]::SFtp
    HostName=$hostname
    PortNumber=$portnumber
    UserName=$username
    Password=$password
 }
 $sessionOptions.AddRawSettings("FSProtocol","2")
 $session = New-Object WinSCP.Session
 If (-Not([string]::IsNullOrEmpty($fingerprint))) 
    {
     $sessionOptions.SshHostKeyFingerprint=$Fingerprint
    }
 If ($detectfingerprint)
    {
     $Fingerprint=Get-SshFingerprint $hostname $portnumber
     $sessionOptions.SshHostKeyFingerprint=$Fingerprint
    }
 If (-Not([string]::IsNullOrEmpty($fingerprint)))  
    {
     Write-Host "Fingerprint = ""$($fingerprint)""" -Foreground Yellow
    }
 Try
    {
     $session.Open($sessionOptions)
     # Download files
     $transferOptions=New-Object WinSCP.TransferOptions
     $transferOptions.TransferMode=[WinSCP.TransferMode]::Binary   
     $RemotePathContent=$session.ListDirectory($remotepath)
     ForEach ($FileInfo In $RemotePathContent.Files)
             {
              If (-Not($FileInfo.IsDirectory)) 
                 {
                  $FilterOk=$FileInfo.Name.ToLower().Contains($Filter.ToLower())
                  If ($FilterOk) 
                     { 
                      $fileurl=$remotepath+$FileInfo.Name
                      $filesize=$FileInfo.Length
                      If (!$localPath.EndsWith("\"))
                         {
                          $localPath+="\"
                         }
                      $transferResult=$session.GetFiles($fileurl,$localPath,$False,$transferOptions)
                      If ($transferResult.IsSuccess) 
                         {
                         If ($Fr) {$S="Téléchargement réussi de ""{0}"" vers ""{1}"" ({2} octets)"} Else {$S="Successfull download of ""{0}"" to ""{1}"" ({2} bytes)"}
                         Write-Host ($S -f $FileUrl, $localpath, $filesize) -foreground "Green"
                         }
                         else
                         {
                         If ($Fr) {$S="Le téléchargement de ""{0}"" vers ""{1}"" a échoué"} Else {$S="Error downloading ""{0}"" to ""{1}"""}
                         Write-Host ($S -f $FileUrl, $localpath) -foreground "Red"
                         }                  
                     }
                     else
                     {
                     If ($Fr) {$S="Le fichier ""{0}"" est ignoré (filtre)"} Else {$S="File ""{0}"" ignored (filter)"}
                     Write-Host ($S -f $FileInfo.Name) -foreground "Yellow"
                     }
                 }
             }
    }
 Finally
   {
   $session.Dispose()
   }
 }
 Catch
 {
 Write-Host "Error: $($_.Exception.Message)"
 }
}
#####################################################################################################################################################################
# UnTarGZ
Function UnTarGZ($TarGZFile,$DestinationFolder)
{
 $null=Expand-7Zip -ArchiveFileName $TarGZFile -TargetPath $Env:TEMP 
 $TarFile=Split-Path $TarGZFile -Leaf -ErrorAction SilentlyContinue
 $TarFile=$Env:TEMP+"\"+$TarFile.Replace(".gz","")
 $null=Expand-7Zip -ArchiveFileName $TarFile.Replace(".gz","") -TargetPath $DestinationFolder
 Remove-Item $TarFile -ErrorAction SilentlyContinue
}
#####################################################################################################################################################################
# ReTarGZ
Function ReTarGZ($OldTarGZFile,$NewTarGZFile)
{
 $null=Expand-7Zip -ArchiveFileName $OldTarGZFile -TargetPath $Env:TEMP 
 $TarFile=Split-Path $OldTarGZFile -Leaf -ErrorAction SilentlyContinue
 $TarFile=$Env:TEMP+"\"+$TarFile.Replace(".gz","")
 $NewTarFile=Split-Path $NewTarGZFile -Leaf -ErrorAction SilentlyContinue
 $NewTarFile=$Env:TEMP+"\"+$NewTarFile.Replace(".gz","")
 Rename-Item -Path $TarFile -NewName $NewTarFile
 $null=Compress-7Zip -Path $NewTarFile -ArchiveFileName $NewTarGZFile -Format GZip
 Remove-Item $OldTarGZFile -ErrorAction SilentlyContinue
 Remove-Item $TarFile -ErrorAction SilentlyContinue
 Remove-Item $NewTarFile -ErrorAction SilentlyContinue
 $S="Renommage de ""{0}"" vers ""{1}""" -f $OldTarGZFile, $NewTarGZFile                    
 Write-Host $S -ForegroundColor "Green"
 Log_Message "RENOMMAGE" $S
}
#####################################################################################################################################################################
# Log Txt / Journal Txt
Function Log_Message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)] $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
#####################################################################################################################################################################
# Date file / Fichier date
Function FileInfoDate
{
Param($Path,$EncodedDate)
$Date=[datetime]::ParseExact($EncodedDate,"yyyyMMdd",$null)
$FileName=("{0}\Bases AAF du {1}.txt" -f $Path, $Date.ToLongDateString())
$Date.ToLongDateString() | Out-File -FilePath $FileName -Encoding Default -Force
}
#####################################################################################################################################################################
# History of archives processed / Historique des archive traitées
Function ProcessedFilesHistory
{
Param($HistoryFileName,$NewFile)
$NewFile | Out-File -FilePath $HistoryFileName -Encoding Default -Append -Force
}
#####################################################################################################################################################################
# Archive already processed ? / Archive déjà traitée
Function AlreadyProcessedFile
{
Param($HistoryFileName,$FileToCheck)
$Content=Get-Content $HistoryFileName -ErrorAction SilentlyContinue | Where-Object {$_.Contains($FileToCheck)}
$Value=![string]::IsNullOrEmpty($Content)
Return $Value
}
#####################################################################################################################################################################
# Bannière
Function Banner
{
Param($MenuLines)
$MarginWidth=2
$MarginChar=" "
$MaxWidth=0;
$BorderTextColor="Magenta"
ForEach ($Line In $MenuLines)
        {
        If ($Line.Length -gt $MaxWidth) {$MaxWidth=$Line.Length}
        }
If ($MaxWidth -gt 0)
   {
   Write-Host "╔" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╗" -ForegroundColor $BorderTextColor
   ForEach ($Line In $MenuLines)
           {
           Write-Host "║" -NoNewline -ForegroundColor $BorderTextColor
           Write-Host ("{0}{1}{2}" -f $MarginChar.PadLeft($MarginWidth),$Line.PadRight($MaxWidth),$MarginChar.PadLeft($MarginWidth)) -NoNewline -ForegroundColor White
           Write-Host "║" -ForegroundColor $BorderTextColor
           }
   Write-Host "╚" -NoNewline -ForegroundColor $BorderTextColor
   For ($i=0; $i -lt $MaxWidth+2*$MarginWidth; $i++)
       {
       Write-Host "═" -NoNewline -ForegroundColor $BorderTextColor
       }
   Write-Host "╝" -ForegroundColor $BorderTextColor
   }
}
#####################################################################################################################################################################
# Main / Principal
$Version="1.3"
$ExecPath=Split-Path -Parent $MyInvocation.MyCommand.Definition 
$DownloadPath=$ExecPath+"\"+$DownloadFolder
$ExtractPath=$ExecPath+"\"+$ExtractFolder
$CSVPath=$ExecPath+"\"+$CSVFolder
$7ZModule=$ExecPath+"\7Z\7Zip4Powershell.dll"
$LogsPath=$ExecPath+"\"+$LogsFolder
$LogFile=$LogsPath+"\"+"ScriptSyncAnnuFed.log"
# Création des dossiers si nécessaire
If (!(Test-Path $DownloadPath)) {$Null=New-Item -ItemType Directory -Force -Path $DownloadPath}
If (!(Test-Path $CSVPath)) {$Null=New-Item -ItemType Directory -Force -Path $CSVPath}
If (!(Test-Path $ExtractPath)) {$Null=New-Item -ItemType Directory -Force -Path $ExtractPath}
If (!(Test-Path $LogsPath)) {$Null=New-Item -ItemType Directory -Force -Path $LogsPath}
# Bienvenue
$BannerText="Synchronisation Mono-domaine de l'Annuaire Académique Fédérateur",
            "(C) 2006-2021 KoXo Developpement",
            "Version $($Version)"
Banner $BannerText
Write-Host
Write-Host ("Dossier des journaux : ""{0}""" -f $LogsPath) -Foregroundcolor "White"
Write-Host
#
# Module 7ZIP4Powershell
#
If ((Get-Module -Name "7Zip4Powershell") -eq $null)
   {
    Write-Host ("Ajout du module PowerShell 7ZIP4Powershell ""{0}""..." -f $7ZModule) -Foregroundcolor "Cyan"    
    $Mod=Import-Module  $7ZModule -PassThru -ErrorAction SilentlyContinue
    If ([string]::IsNullOrEmpty($Mod)) 
         {
         $S="Le module PowerShell 7ZIP4Powershell n'est pas présent !"
         Write-Host $S -Foregroundcolor "Red"
         Write-Host "Press any key ..."
         Read-Host
         Exit
         }
         else
         {
         $S="Le module Powershell 7ZIP4Powershell est présent et chargé !"
         Write-Host $S -Foregroundcolor "Green"
         }
    Write-Host
   }
If ($TranfertProtocole.ToUpper() -eq "FTP")
   {
    #
    # FTP
    #
    Write-Host ("Synchronisation FTP ""{0}"" -> ""{1}""" -f $FTPUrl, $DownloadPath) -Foregroundcolor "Cyan"
    Log_Message "SYNCHRO-FTP" (""""+$FTPUrl+""" -> """+$DownloadPath+"""")
    $Credentials=New-Object System.Net.NetworkCredential($Username,$Password) 
    DownloadFtpDirectory $FTPUrl $Credentials $DownloadPath $Filter $False $False
    }
If ($TranfertProtocole.ToUpper() -eq "FTP/S")
   {
    #
    # FTP/SSL
    #
    Write-Host ("Synchronisation FTP/SSL ""{0}"" -> ""{1}""" -f $FTPUrl, $DownloadPath) -Foregroundcolor "Cyan"
    Log_Message "SYNCHRO-FTP/S" (""""+$FTPUrl+""" -> """+$DownloadPath+"""")
    $Credentials=New-Object System.Net.NetworkCredential($Username,$Password) 
    DownloadFtpDirectory $FTPUrl $Credentials $DownloadPath $Filter $False $True
    }
If ($TranfertProtocole.ToUpper() -eq "SFTP")
   {
    #
    # SFTP
    #
    Write-Host ("Synchronisation SFTP ""{0}:{1}"" -> ""{2}""" -f $SFTPHostname, $SFTPPortNumber, $DownloadPath) -Foregroundcolor "Cyan"
    Log_Message "SYNCHRO-SFTP" (""""+$SFTPHostname+""" -> """+$DownloadPath+"""")
    DownloadSFTPDirectory $SFTPHostname $SFTPPortNumber $Username $Password $SFTPDetectFingerPrint $SFTPFingerPrint $SFTPRemotePath $Downloadpath $Filter
    }
#
# Décompression du fichier le plus récent
# 
$MyFolder=Get-ChildItem -Path $DownloadPath | Where-Object {!$_.PSIsContainer}
If ($Myfolder.Count -gt 0) 
   {
    $Dates=New-Object System.Collections.ArrayList
    ForEach ($MyFile In $MyFolder)
            {
              # Si le nom contient "_Complet_" on suppose que la date est contenue dans la dernière partiue du nom (le caractère "_" étant le séparateur).
              # Nom du genre "Périmètre_Complet_20170605.tar.gz" ou "Périmètre_ENT_Complet_20170605.tar.gz" 
              # Sinon le nomne contient pas "_Complet_" alors c'est la date de la dernière écriture qui est prise en compte et l'archive est reconstruite
              #  Exemple de nom "Périmètre_Complet_20170605.tar.gz"
              If ($Myfile.Name.ToString().ToLower().Contains("_complet_"))
                 {
                  $FileArr=$MyFile.Name.Split("_")
                  $FileDate=$FileArr[$FileArr.Count-1].Split(".")[0]
                  $null=$Dates.Add($FileDate+";"+$MyFile.Name)
                 }
                 else
                 {
                  $FileDate=$Myfile.LastWriteTime.Date.ToString("yyyyMMdd")
                  # Renommage du TarGz avec horodatage dans le nom
                  $TarGZFolder=Split-Path -Path $Myfile.VersionInfo.FileName -Parent -ErrorAction SilentlyContinue
                  $NewTarGZ="{0}\{1}_Complet_{2}.tar.gz" -f $TarGZFolder, $MyFile.Name.Split(".")[0], $FileDate
                  ReTarGZ $MyFile.VersionInfo.FileName $NewTarGZ
                  $NewTarGZ=Split-Path -Path $NewTarGZ -Leaf -ErrorAction SilentlyContinue
                  $null=$Dates.Add($FileDate+";"+$NewTarGZ)
                 }
             }
    $Dates=$Dates | Sort-Object -Unique -Descending    
    If ($Dates.Count -gt 1) 
       {
        $Date0=$Dates[0]
       }
       else
       {
        $Date0=$Dates       
       }
    $LastFile=$DownloadPath+"\"+$Date0.Split(";")[1]        
    $EncodedDate=$Date0.Split(";")[0]
    Write-Host "Archive la plus récente : " -ForegroundColor "White"
    Write-Host ("Fichier : ""{0}"" ({1})" -f $LastFile, $EncodedDate) -ForegroundColor "Green"
    Write-Host
    #
    # Archive déjà traitée ?
    #
    $AlreadyProcessed=AlreadyProcessedFile ($ExecPath+"\"+$FilesHistoryFileName) $Date0
        If ($AlreadyProcessed)
           {
            $S=("L'archive ""{0}"" ({1}) a déjà été traitée !" -f $LastFile, $Date0.Split(";")[0])
            Log_Message "ATTENTION" $S
            Write-Host $S -Foregroundcolor "Yellow"
            Exit
           }
    ProcessedFilesHistory ($ExecPath+"\"+$FilesHistoryFileName) $Date0
    Remove-Item ($ExtractPath+'\*') -Recurse -ErrorAction SilentlyContinue
    Log_Message "DECOMPRESSION-TAR-GZ" (""""+$LastFile+""" -> """+$ExtractPath+"""")
    UnTarGZ $LastFile $ExtractPath
    # Fichiers XML
    Write-Host "Fichiers XML décompressés : " -Foregroundcolor "White"
    $XMLFiles=Get-ChildItem -Path $ExtractPath | Where-Object {!$_.PSIsContainer -AND $_.Extension -like ".xml"}
    If ($XMLFiles.Count -gt 0) 
       {
        ForEach ($XMLFile In $XMLFiles)
                {
                 If ($XMLFile.Name.IndexOf("_Etab") -gt 0) 
                    { 
                     $FichieEtablissement=$XMLFile 
                    }
                 If ($XMLFile.Name.IndexOf("_Eleve") -gt 0) 
                    { 
                     $FichierEleve=$XMLFile 
                    }
                 If ($XMLFile.Name.IndexOf("_PersEducNat") -gt 0) 
                    { 
                     $FichierPersEducNat=$XMLFile 
                    }
                } 
        $S=("Fichier ""Eleve"" : ""{0}""" -f $FichierEleve.FullName)
        Log_Message "XML" $S
        Write-Host $S -Foregroundcolor "Green"
        $S=("Fichier ""PersEducNat"" : ""{0}""" -f $FichierPersEducNat.FullName)
        Log_Message "XML" $S
        Write-Host $S -Foregroundcolor "Green"
        $S=("Fichier ""Etablissement"" : ""{0}""" -f $FichieEtablissement.FullName)
        Log_Message "XML" $S
        Write-Host $S -Foregroundcolor "Green"
        Write-Host
        Remove-Item ($CSVPath+'\*') -Recurse -ErrorAction SilentlyContinue
        $AAFConvertTool=$ExecPath+"\"+"AnnuFedXmlCsv.exe"
        If ((Test-Path $AAFConvertTool))
           { 
            $AAFParameters="/CONVERSION /ELEVES=""{0}"" /PERSONNELS=""{1}"" /ETABLISSEMENTS=""{2}"" /DESTINATION=""{3}"" /JOURNAUX=""{4}"""
            If ($Accents) 
               {
                $AAFParameters=$AAFParameters+" /ACCENTS_ELEVES=1 /ACCENTS_ADMINISTRATIFS=1 /ACCENTS_PROFESSEURS=1"
               }
               else
               {
                $AAFParameters=$AAFParameters+" /ACCENTS_ELEVES=0 /ACCENTS_ADMINISTRATIFS=0 /ACCENTS_PROFESSEURS=0"
               }
            If ($SupprimerDoublesTirets) 
               {
                $AAFParameters=$AAFParameters+" /SUPPRIMER_DOUBLES_TIRETS=1"
               }
               else
               {
                $AAFParameters=$AAFParameters+" /SUPPRIMER_DOUBLES_TIRETS=0"
               }
            If ($AttachementStructureOnly) 
               {
                $AAFParameters=$AAFParameters+" /ETABLISSEMENT_RATTACHEMENT_SEULEMENT=1"
               }
               else
               {
                $AAFParameters=$AAFParameters+" /ETABLISSEMENT_RATTACHEMENT_SEULEMENT=0"
               }
            If ($AddDateToCSvFiles) 
               {
                $AAFParameters=$AAFParameters+" /AJOUTER_LA_DATE_AUX_FICHIERS_CSV=1"
               }
               else
               {
                $AAFParameters=$AAFParameters+" /AJOUTER_LA_DATE_AUX_FICHIERS_CSV=0"
               }
            $AAFParameters=($AAFParameters -f $FichierEleve.FullName, $FichierPersEducNat.FullName, $FichieEtablissement.FullName, $CSVPath, $LogsPath)
            $AAFParameters=$AAFParameters+" "+$MultiSitesParameters
            $S=("Appel de l'outil de conversion Xml vers Csv ""AnnuFedXmlCsv"" : ""{0}""" -f $AAFConvertTool)
            Write-Host $S -Foregroundcolor "White"
            Log_Message "EXECUTABLE" $S
            Log_Message "PARAMETRES" $AAFParameters
            Write-Host "Patienter ..." -Foregroundcolor "Cyan" -NoNewline
            Start-Process -Wait $AAFConvertTool $AAFParameters
            $CSVFiles=Get-ChildItem -Path $CSVPath | Where-Object {!$_.PSIsContainer -AND $_.Extension -like ".csv"}
            $S=("Nombre de fichiers CSV générés dans le dossier ""{0}"" : {1}" -f $CSVPath, $CSVFiles.Count)
            Write-Host ("`r"+$S) -Foregroundcolor "Yellow" 
            # Ajout du fichier texte d'info de date des bases 
            FileInfoDate $CSVPath $EncodedDate
            # Renommage des fichiers CSV
            $CSVFiles=Get-ChildItem -Path $CSVPath | Where-Object {!$_.PSIsContainer -AND $_.Extension -like ".csv"}
            If ($CSVFiles.Count -gt 0) 
               {
                ForEach ($CSVFile In $CSVFiles)
                        {
                         $CSVNewName=$CSVFile.Name.Split("_")[1]
                         Rename-Item -Path $CSVFile.FullName -NewName $CSVNewName
                         $S="Renommage de ""{0}"" vers ""{1}""" -f $CSVFile.FullName, $CSVNewName                    
                         Write-Host $S -ForegroundColor "Green"
                         Log_Message "RENOMMAGE" $S
                        }
               }
           }
           else
           {
            $S=("Le fichier de conversion ""{0}"" est introuvable!" -f $AAFConvertTool)
            Write-Host $S -Foregroundcolor "Red"
            Log_Message "ERREUR" $S
           }
       }

   }
#####################################################################################################################################################################
